#!/usr/bin/env python
from numpy import *
from scipy.special import airy
from scipy.optimize import fsolve
from matplotlib.pyplot import *


# Anfangswerte
iv = array([ 0.35502805388781723926,
            -0.25881940379280679841])

# Exakte Loesung
exact = lambda t: row_stack(airy(t)[:2])

# Intervall
T = -40.0


###################
#  Teilaufgabe 1  #
###################

# Airy rechte Seite
rhs = lambda t, y: array([  0,
                            0])


####################
#  Teilaufgabe 2   #
####################

def integrate_EE(y0, tstart, tend, N):
    r"""Integrate ODE with explicit Euler method

    Input: y0     ... initial condition
           tstart ... start t
           tend   ... end   t
           N      ... number of steps

    Output: t ... variable
            y ... solution
    """
    y = zeros((2,N+1))
    t = zeros((N+1,))
    t[0] = tstart
    y[:,0] = y0
    h = (tend - tstart) / float(N)

    # TODO: t, y berechnen
    return t, y


def integrate_IE(y0, tstart, tend, N):
    r"""Integrate ODE with implicit Euler method

    Input: y0     ... initial condition
           tstart ... start t
           tend   ... end   t
           N      ... number of steps

    Output: t ... variable
            y ... solution
    """
    y = zeros((2,N+1))
    t = zeros((N+1,))
    t[0] = tstart
    y[:,0] = y0
    h = (tend - tstart) / float(N)

    # todo: t, y berechnen
    return t, y


def integrate_EM(y0, tstart, tend, N):
    r"""Integrate ODE with explicit midpoint method

    Input: y0     ... initial condition
           tstart ... start t
           tend   ... end   t
           N      ... number of steps

    Output: t ... variable
            y ... solution
    """
    y = zeros((2,N+1))
    t = zeros((N+1,))
    t[0] = tstart
    y[:,0] = y0
    h = (tend - tstart) / float(N)

    # TODO: t,y berechnen
    return t, y


def integrate_IM(y0, tstart, tend, N):
    r"""Integrate ODE with implicit midpoint method

    Input: y0     ... initial condition
           tstart ... start t
           tend   ... end   t
           N      ... number of steps

    Output: t ... variable
            y ... solution
    """
    y = zeros((2,N+1))
    t = zeros((N+1,))
    t[0] = tstart
    y[:,0] = y0
    h = (tend - tstart) / float(N)

    # todo: t, y berechnen
    return t, y


N = 10**3
t, y = integrate_EE(iv, 0.0, T, N)
figure()
plot(t, exact(t)[0], "r")
plot(t, y[0,:], "b")
#plot(t, y[1,:], "g")
grid(True)
xlabel("t")
ylabel("Ai(t)")
savefig("img/ai_expl_euler.png")

t, y = integrate_IE(iv, 0.0, T, N)
figure()
plot(t, exact(t)[0], "r")
plot(t, y[0,:], "b")
#plot(t, y[1,:], "g")
grid(True)
xlabel("t")
ylabel("Ai(t)")
savefig("img/ai_impl_euler.png")

t, y = integrate_EM(iv, 0.0, T, N)
figure()
plot(t, exact(t)[0], "r")
plot(t, y[0,:], "b")
#plot(t, y[1,:], "g")
grid(True)
xlabel("t")
ylabel("Ai(t)")
savefig("img/ai_expl_midpoint.png")

t, y = integrate_IM(iv, 0.0, T, N)
figure()
plot(t, exact(t)[0], "r")
plot(t, y[0,:], "b")
#plot(t, y[1,:], "g")
grid(True)
xlabel("t")
ylabel("Ai(t)")
savefig("img/ai_impl_midpoint.png")


####################
#  Teilaufgabe 3   #
####################


def RK(B, y0, tstart, tend, N):
    r"""
    Integrate the equation with a Runge-Kutta rule.

    Input: B      ... Butcher Schema
           y0     ... initial condition
           tstart ... start t
           tend   ... end   t
           N      ... number of steps

    Output: t ... variable
            y ... solution
    """
    y = zeros((2,N+1))
    t = zeros((N+1,))
    t[0] = tstart
    y[:,0] = y0
    h = (tend - tstart) / float(N)
    # TODO: t, y berechnen
    return t, y


B = array([[0, 0],
           [0, 1]])

t, y = RK(B, iv, 0.0, T, N)
figure()
plot(t, exact(t)[0], "r")
plot(t, y[0,:], "b")
#plot(t, y[1,:], "g")
grid(True)
xlabel("t")
ylabel("Ai(t)")
savefig("img/ai_RK_expl_Euler.png")


B = array([[1, 1],
           [0, 1]])

t, y = RK(B, iv, 0.0, T, N)
figure()
plot(t, exact(t)[0], "r")
plot(t, y[0,:], "b")
#plot(t, y[1,:], "g")
grid(True)
xlabel("t")
ylabel("Ai(t)")
savefig("img/ai_RK_impl_Euler.png")


B = array([[ 0, 0, 0],
           [.5,.5, 0],
           [ 0, 0, 1]])

t, y = RK(B, iv, 0.0, T, N)
figure()
plot(t, exact(t)[0], "r")
plot(t, y[0,:], "b")
#plot(t, y[1,:], "g")
grid(True)
xlabel("t")
ylabel("Ai(t)")
savefig("img/ai_RK_expl_midpoint.png")


B = array([[.5,.5],
           [ 0, 1]])

t, y = RK(B, iv, 0.0, T, N)
figure()
plot(t, exact(t)[0], "r")
plot(t, y[0,:], "b")
#plot(t, y[1,:], "g")
grid(True)
xlabel("t")
ylabel("Ai(t)")
savefig("img/ai_RK_impl_midpoint.png")

####################
#  Teilaufgabe 4   #
####################

def RK_38(y0, tstart, tend, N):
    r"""
    Integrate the equation with the 3/8 Runge-Kutta rule.

    Input:
           y0     ... initial condition
           tstart ... start t
           tend   ... end   t
           N      ... number of steps

    Output: t ... variable
            y ... solution
    """
    # TODO: t, y berechnen (HINT: die Funktion RK)

    # DELETE LINE BELOW
    return linspace(tstart,tend,N), zeros((y0.shape[0], N))
    # DELETE LINE ABOVE


t, y = RK_38(iv, 0.0, T, N)
figure()
plot(t, exact(t)[0], "r")
plot(t, y[0,:], "b")
#plot(t, y[1,:], "g")
grid(True)
xlabel("t")
ylabel("Ai(t)")
savefig("img/ai_RK_38.png")
